@@ -1,6 +1,7 @@
-#' Loads in and converts input AIRR-compatible tsv file(s) into the Platypus VGM object format.
-#' @description All compulsory AIRR data columns are needed. Additionally, the following columns are required: v_call, cell_id, clone_id. If trim.and.align is set to TRUE additionally the following columns are needed: v_sequence_start, j_sequence_end.
-#' Note on TRUST4 input: TRUST4 (https://doi.org/10.1038/s41592-021-01142-2) is a newly alignment tool for VDJ data by the Shirley lab. It is able to also extract VDJ sequences from 10x GEX data. We are actively testing TRUST4 as an alternative to Cellranger and can not give recommendations as of now. This function does support the conversion of TRUST4 airr output data into the Platypus VGM format. In that case, an extra column will be added describing whether the full length VDJ sequence was extracted for any given cell and chain.
+#'AIRR to Platypus V3 VGM compatibility function
+#'
+#'@description Loads in and converts input AIRR-compatible tsv file(s) into the Platypus VGM object format.All compulsory AIRR data columns are needed. Additionally, the following columns are required: v_call, cell_id, clone_id. If trim.and.align is set to TRUE additionally the following columns are needed: v_sequence_start, j_sequence_end.
+#' Note on TRUST4 input: TRUST4 (https://doi.org/10.1038/s41592-021-01142-n2) is a newly alignment tool for VDJ data by the Shirley lab. It is able to also extract VDJ sequences from 10x GEX data. We are actively testing TRUST4 as an alternative to Cellranger and can not give recommendations as of now. This function does support the conversion of TRUST4 airr output data into the Platypus VGM format. In that case, an extra column will be added describing whether the full length VDJ sequence was extracted for any given cell and chain.
 #'@param AIRR.input Source of the AIRR table(s) as a list. There are 2 available input options: 1. 1. List with local paths to .tsv files / 3. List of AIRR tables loaded in as R objects within the current R environment.
 #'@param get.VDJ.stats Boolean. Defaults to TRUE. Whether to generate summary statistics on repertoires and output those as output_VGM[[3]]
 #'@param VDJ.combine Boolean. Defaults to TRUE. Whether to integrate repertoires. A sample identifier will be appended to each barcode both. Highy recommended for all later functions
@@ -60,13 +61,13 @@ PlatypusDB_AIRR_to_VGM <- function(AIRR.input,
     #If more than two contigs of one chain (e.g. 2 TRB) are present, the elements will be pasted separated by a ";" into the relevant fields (in the case of TRB, into the Hb columns)
 
     #Get number of chains
-    HC_count <- sum(stringr::str_count(curr.contigs$v_call, pattern = "(TRB|IGH)"))
-    LC_count <- sum(stringr::str_count(curr.contigs$v_call, pattern = "(TRA|IG(K|L))"))
+    HC_count <- sum(stringr::str_count(curr.contigs$v_call, pattern = "(TRG|TRB|IGH)"))
+    LC_count <- sum(stringr::str_count(curr.contigs$v_call, pattern = "(TRD|TRA|IGK|IGL)"))
 
     #In this case we need to make much less effort with pasting together, so we can save time
     if(HC_count == 1 & LC_count == 1){
 
-      if(which(stringr::str_detect(curr.contigs$v_call, "(TRA|IG(K|L))")) == 1){ #make row 1 the heavy chain in case it is not already
+      if(which(stringr::str_detect(curr.contigs$v_call, "(TRD|TRA|IGK|IGL)")) == 1){ #make row 1 the heavy chain in case it is not already
         curr.contigs <- curr.contigs[c(2,1),]}
 
       #fill in the pasted info to curr.barcode directly
@@ -108,27 +109,28 @@ PlatypusDB_AIRR_to_VGM <- function(AIRR.input,
     } else { # this for cells with aberrant chain numbers
 
       contigs_pasted <- stats::setNames(data.frame(matrix(ncol = ncol(curr.contigs), nrow = length(unique(curr.contigs$v_call)))), names(curr.contigs)) #the dataframe may be one or two rows too long, this will not matter / ROW 1 = Heavy chain information / ROW 2 = Light chain information. This order is maintained even if one of the two chains is not present!
+      if(nrow(contigs_pasted) == 3) contigs_pasted <- contigs_pasted[-3,]
 
       #Heavy/b chain count
       if(HC_count == 1){
-        contigs_pasted[1,] <- curr.contigs[stringr::str_detect(curr.contigs$v_call, pattern = "(TRB|IGH)"),]
+        contigs_pasted[1,] <- curr.contigs[stringr::str_detect(curr.contigs$v_call, pattern = "(TRG|TRB|IGH)"),]
       } else if(HC_count == 0){
         contigs_pasted[1,] <- ""
       } else if(HC_count > 1){
         for(k in 1:ncol(curr.contigs)){
-          contigs_pasted[1,k] <- paste0(curr.contigs[which(stringr::str_detect(curr.contigs$v_call, pattern = "(TRB|IGH)")), k], collapse = ";")
+          contigs_pasted[1,k] <- paste0(curr.contigs[which(stringr::str_detect(curr.contigs$v_call, pattern = "(TRG|TRB|IGH)")), k], collapse = ";")
         }
       }
       ### Order of CDRs with multiple chains is determined here
 
       #Light/a chain count
       if(LC_count == 1){
-        contigs_pasted[2,] <- curr.contigs[stringr::str_detect(curr.contigs$v_call, pattern = "(TRA|IG(K|L))"),]
+        contigs_pasted[2,] <- curr.contigs[stringr::str_detect(curr.contigs$v_call, pattern = "(TRD|TRA|IGK|IGL)"),]
       } else if(LC_count == 0){
         contigs_pasted[2,] <- ""
       } else if(LC_count > 1){
         for(k in 1:ncol(curr.contigs)){
-          contigs_pasted[2,k]  <- paste0(curr.contigs[which(stringr::str_detect(curr.contigs$v_call, pattern = "(TRA|IG(K|L))")),k],collapse = ";")
+          contigs_pasted[2,k]  <- paste0(curr.contigs[which(stringr::str_detect(curr.contigs$v_call, pattern = "(TRD|TRA|IGK|IGL)")),k],collapse = ";")
         }
       }
 
@@ -141,11 +143,9 @@ PlatypusDB_AIRR_to_VGM <- function(AIRR.input,
       } else if(stringr::str_detect(contigs_pasted$v_call[1], "IG") | stringr::str_detect(contigs_pasted$v_call[2], "IG")) {curr.barcode$celltype <- "B cell"
       } else {curr.barcode$celltype <- "Unkown"}
 
-
       curr.barcode$Nr_of_VDJ_chains <- HC_count
       curr.barcode$Nr_of_VJ_chains <- LC_count
 
-
       curr.barcode$VDJ_cdr3s_aa <- contigs_pasted$junction_aa[1]
       curr.barcode$VJ_cdr3s_aa <- contigs_pasted$junction_aa[2]
       curr.barcode$VDJ_cdr3s_nt <- contigs_pasted$junction[1]
@@ -176,12 +176,14 @@ PlatypusDB_AIRR_to_VGM <- function(AIRR.input,
     #CHECK IF THERE IS 1 2 or 0 chains to process
     if(HC_count == 1){
 
+      HC_index <- which(stringr::str_detect(curr.contigs$v_call, "(TRG|TRB|IGH)"))
+
       #extract match
-      curr.barcode$VDJ_sequence_nt_raw <- curr.contigs$sequence[1]
+      curr.barcode$VDJ_sequence_nt_raw <- curr.contigs$sequence[HC_index]
       if(trim.and.align){
-      curr.barcode$VDJ_sequence_nt_trimmed <- substr(curr.barcode$VDJ_sequence_nt_raw, as.numeric(curr.contigs$v_sequence_start[1]), as.numeric(curr.contigs$j_sequence_end[1])-1)
-      curr.barcode$VDJ_sequence_aa <- curr.contigs$sequence_aa[1]
-      curr.barcode$VDJ_trimmed_ref <- substr(curr.contigs$germline_alignment[1], as.numeric(curr.contigs$v_sequence_start[1]), as.numeric(curr.contigs$j_sequence_end[1])-1)
+      curr.barcode$VDJ_sequence_nt_trimmed <- substr(curr.barcode$VDJ_sequence_nt_raw, as.numeric(curr.contigs$v_sequence_start[HC_index]), as.numeric(curr.contigs$j_sequence_end[HC_index]))
+      curr.barcode$VDJ_sequence_aa <- curr.contigs$sequence_aa[HC_index]
+      curr.barcode$VDJ_trimmed_ref <- substr(curr.contigs$germline_alignment[HC_index], as.numeric(curr.contigs$v_sequence_start[HC_index]), as.numeric(curr.contigs$j_sequence_end[HC_index]))
       } else {
         curr.barcode$VDJ_sequence_nt_trimmed <- ""
         curr.barcode$VDJ_sequence_aa <- ""
@@ -201,18 +203,18 @@ PlatypusDB_AIRR_to_VGM <- function(AIRR.input,
       to_paste_trimmed <- c()
       to_paste_aa <- c()
       to_paste_ref_trimmed <- c()
-      HC_rows <- curr.contigs[stringr::str_detect(curr.contigs$v_call, "(TRB|IGH)"),]
+      HC_rows <- curr.contigs[stringr::str_detect(curr.contigs$v_call, "(TRG|TRB|IGH)"),]
       #looping contigs in annotation
       for(l in 1:nrow(HC_rows)){
             #get sequence
             to_paste <- append(to_paste, HC_rows$sequence[l])
             if(trim.and.align == T){
 
-              to_paste_trimmed <- c(to_paste_trimmed, substr(HC_rows$sequence[l], as.numeric(HC_rows$v_sequence_start[l]), as.numeric(HC_rows$j_sequence_end[l])-1))
+              to_paste_trimmed <- c(to_paste_trimmed, substr(HC_rows$sequence[l], as.numeric(HC_rows$v_sequence_start[l]), as.numeric(HC_rows$j_sequence_end[l])))
 
               to_paste_aa <- c(to_paste_aa, HC_rows$sequence_aa[l])
 
-              to_paste_ref_trimmed <- c(to_paste_ref_trimmed, substr(HC_rows$germline_alignment[l], as.numeric(HC_rows$v_sequence_start[l]), as.numeric(HC_rows$j_sequence_end[l])-1))
+              to_paste_ref_trimmed <- c(to_paste_ref_trimmed, substr(HC_rows$germline_alignment[l], as.numeric(HC_rows$v_sequence_start[l]), as.numeric(HC_rows$j_sequence_end[l])))
 
             } else {
               to_paste_trimmed <- ""
@@ -230,12 +232,14 @@ PlatypusDB_AIRR_to_VGM <- function(AIRR.input,
     #Light/a
     if(LC_count == 1){
 
+      LC_index <- which(stringr::str_detect(curr.contigs$v_call, "(TRD|TRA|IGK|IGL)"))
+
       #extract match
-      curr.barcode$VJ_sequence_nt_raw <- curr.contigs$sequence[2]
+      curr.barcode$VJ_sequence_nt_raw <- curr.contigs$sequence[LC_index]
       if(trim.and.align){
-        curr.barcode$VJ_sequence_nt_trimmed <- substr(curr.barcode$VJ_sequence_nt_raw, as.numeric(curr.contigs$v_sequence_start[2]), as.numeric(curr.contigs$j_sequence_end[2])-1)
-        curr.barcode$VJ_sequence_aa <- curr.contigs$sequence_aa[2]
-        curr.barcode$VJ_trimmed_ref <- substr(curr.contigs$germline_alignment[2], as.numeric(curr.contigs$v_sequence_start[2]), as.numeric(curr.contigs$j_sequence_end[2])-1)
+        curr.barcode$VJ_sequence_nt_trimmed <- substr(curr.barcode$VJ_sequence_nt_raw, as.numeric(curr.contigs$v_sequence_start[LC_index]), as.numeric(curr.contigs$j_sequence_end[LC_index]))
+        curr.barcode$VJ_sequence_aa <- curr.contigs$sequence_aa[LC_index]
+        curr.barcode$VJ_trimmed_ref <- substr(curr.contigs$germline_alignment[LC_index], as.numeric(curr.contigs$v_sequence_start[2]), as.numeric(curr.contigs$j_sequence_end[LC_index]))
       } else {
         curr.barcode$VJ_sequence_nt_trimmed <- ""
         curr.barcode$VJ_sequence_aa <- ""
@@ -248,25 +252,25 @@ PlatypusDB_AIRR_to_VGM <- function(AIRR.input,
       curr.barcode$VJ_sequence_aa <- ""
       curr.barcode$VJ_trimmed_ref <- ""
 
-    } else if(LC_count > 1){ #MORE THAN ONE HC
+    } else if(LC_count > 1){ #MORE THAN ONE LC
       #from the annotations extract sequence and paste
       #Heavy/b
       to_paste <- c()
       to_paste_trimmed <- c()
       to_paste_aa <- c()
       to_paste_ref_trimmed <- c()
-      LC_rows <- curr.contigs[stringr::str_detect(curr.contigs$v_call, "(TRA|IG(K|L))"),]
+      LC_rows <- curr.contigs[stringr::str_detect(curr.contigs$v_call, "(TRD|TRA|IGK|IGL)"),]
       #looping contigs in annotation
       for(l in 1:nrow(LC_rows)){
         #get sequence
         to_paste <- append(to_paste, LC_rows$sequence[l])
         if(trim.and.align == T){
 
-          to_paste_trimmed <- c(to_paste_trimmed, substr(LC_rows$sequence[l], as.numeric(LC_rows$v_sequence_start[l]), as.numeric(LC_rows$j_sequence_end[l])-1))
+          to_paste_trimmed <- c(to_paste_trimmed, substr(LC_rows$sequence[l], as.numeric(LC_rows$v_sequence_start[l]), as.numeric(LC_rows$j_sequence_end[l])))
 
           to_paste_aa <- c(to_paste_aa, LC_rows$sequence_aa[l])
 
-          to_paste_ref_trimmed <- c(to_paste_ref_trimmed, substr(LC_rows$germline_alignment[l], as.numeric(LC_rows$v_sequence_start[l]), as.numeric(LC_rows$j_sequence_end[l])-1))
+          to_paste_ref_trimmed <- c(to_paste_ref_trimmed, substr(LC_rows$germline_alignment[l], as.numeric(LC_rows$v_sequence_start[l]), as.numeric(LC_rows$j_sequence_end[l])))
 
         } else {
           to_paste_trimmed <- ""
@@ -298,8 +302,8 @@ PlatypusDB_AIRR_to_VGM <- function(AIRR.input,
       #gsub to be able to process TCRs as well
       airr.list[[k]]$chain <- substr(airr.list[[k]]$v_call, 1,3)
 
-      airr.list[[k]]$chain <- gsub(pattern = "TRB", replacement = "IGH", airr.list[[k]]$chain)
-      airr.list[[k]]$chain <- gsub(pattern = "TRA", replacement = "IGL", airr.list[[k]]$chain)
+      airr.list[[k]]$chain <- gsub(pattern = "TRB|TRG", replacement = "IGH", airr.list[[k]]$chain)
+      airr.list[[k]]$chain <- gsub(pattern = "TRA|TRD", replacement = "IGL", airr.list[[k]]$chain)
 
       #info on sample
       VDJ.stats[length(VDJ.stats)+1] <- paste0("s", k)
@@ -385,7 +389,7 @@ PlatypusDB_AIRR_to_VGM <- function(AIRR.input,
 
 
   if(verbose) cat("\n Loading in data     ")
-  print(Sys.time())
+  if(verbose) print(Sys.time())
 
   #combine all samples into one table
   if(missing(get.VDJ.stats)) get.VDJ.stats <- T
@@ -395,14 +399,14 @@ PlatypusDB_AIRR_to_VGM <- function(AIRR.input,
 
   vdj_loaded <- F
   #get input as list
-  if(class(AIRR.input) == "list"){
-    if(class(AIRR.input[[1]]) == "data.frame"){ #case 1.
+  if(inherits(AIRR.input,"list")){
+    if(inherits(AIRR.input[[1]],"data.frame")){ #case 1.
       if(verbose) cat("\n Dataframe input detected")
       airr.list <- AIRR.input
       airr.names <- "Input from R enviroment"
       vdj_loaded <- T
 
-    } else if(class(AIRR.input[[1]]) == "character"){ #case 3
+    } else if(inherits(AIRR.input[[1]],"character")){ #case 3
       if(verbose) cat("\n Local paths input detected. Loading in tables")
       airr.names <- paste0(unlist(AIRR.input),collapse = ";")
       vdj_load_error <- tryCatch({
@@ -451,7 +455,7 @@ PlatypusDB_AIRR_to_VGM <- function(AIRR.input,
                      "group.id")
 
   if(verbose) cat("\n AIRR tables loaded     ")
-  print(Sys.time())
+  if(verbose) print(Sys.time())
 
   stats.done <- F
   if(get.VDJ.stats == T){
@@ -495,8 +499,27 @@ PlatypusDB_AIRR_to_VGM <- function(AIRR.input,
     VDJ.proc.list <- list()
     for(i in 1:length(airr.list)){
 
-      if(verbose) cat(paste0("\n Starting VDJ barcode iteration ", i , " of ", length(airr.list), "...     "))
-      print(Sys.time())
+      if(verbose) cat(paste0("\n Starting VDJ barcode iteration ", i , " of ", length(airr.list), "...    "))
+      if(verbose) print(Sys.time())
+
+      if(trim.and.align){#validate that start end end columns of V segments are available and numeric
+
+        if(any(!c("v_cigar", "j_cigar") %in% names(airr.list[[i]]))){
+          trim.and.align <- F
+          warning('Columns "v_cigar", "j_cigar" for trimming information where not found in current airr input table. Setting trim.and.align to FALSE')
+        }
+        if(all(c("v_cigar", "j_cigar") %in% names(airr.list[[i]]))) {
+          #use CIGAR string for trimming
+          #Assume standard CIGAR output: ...S...M...S (first number indicates soft clipped nucleotides before alignment, second number indicates matching nuclotides, third number indicate soft clipped nucleotides at the end of the sequence)
+          airr.list[[i]]$v_sequence_start <- as.numeric(gsub("S","",stringr::str_extract(airr.list[[i]]$v_cigar, "^\\d+S")))+1 #first soft clipped value
+          #This returns NA if there is no clipped sequence and the alignment starts from the first nucleotide. In that case we need no trimming
+          airr.list[[i]]$v_sequence_start[is.na(airr.list[[i]]$v_sequence_start)] <- 0
+
+          airr.list[[i]]$j_sequence_end <- nchar(airr.list[[i]]$sequence) - as.numeric(gsub("S","",stringr::str_extract(airr.list[[i]]$j_cigar, "\\d+S$"))) #last soft clipped value
+          #This returns NA if there is no clipped sequence at the end and the cigar ends in ...M In that case we do not trim from this side and set the end to the length of the contig
+          airr.list[[i]]$j_sequence_end[is.na(airr.list[[i]]$j_sequence_end)] <- nchar(airr.list[[i]]$sequence)[is.na(airr.list[[i]]$j_sequence_end)]
+        }
+      }
 
       VDJ.proc.list[[i]] <- lapply(barcodes_VDJ[[i]], AIRR_barcode_VDJ_iteration, airrs = airr.list[[i]])
 
@@ -515,7 +538,6 @@ PlatypusDB_AIRR_to_VGM <- function(AIRR.input,
       VDJ.proc.list[[i]]$group_id <- group.id[i]
 
       #add frequency column (i.e. all cells in clonotype2 will have the same entry, that is the number of cells in clonotype2)
-      print(names(VDJ.proc.list[[i]]))
 
       if("clonotype_id" %in% names(VDJ.proc.list[[i]])){
         VDJ.proc.list[[i]]$clonotype_id <- "Undetermined"
@@ -536,7 +558,7 @@ PlatypusDB_AIRR_to_VGM <- function(AIRR.input,
       airr.list[[i]] <- airr.list[[i]][,-c(which(names(airr.list[[i]]) %in% c("sequence", "sequence_aa", "sequence_id", "clone_id", "v_call", "j_call", "d_call","c_call","productive", "junction", "junction_aa", "sequence_alignment", "germline_alignment", "is_cell", "consensus_count", "complete_vdj")))]
 
       #VJ
-      VJ_airr <- subset(airr.list[[i]], stringr::str_detect(airr.list[[i]]$chain_for_merge, "(TRA|IG(K|L))"))
+      VJ_airr <- subset(airr.list[[i]], stringr::str_detect(airr.list[[i]]$chain_for_merge, "(TRD|TRA|IG(K|L))"))
       VJ_airr <- VJ_airr[,-c(which(names(VJ_airr) %in% c("chain_for_merge")))]
       #rename to fit VGM
       names(VJ_airr) <- paste0("VJ_", names(VJ_airr))
@@ -548,7 +570,7 @@ PlatypusDB_AIRR_to_VGM <- function(AIRR.input,
 
 
       #VDJ
-      VDJ_airr <- subset(airr.list[[i]], stringr::str_detect(airr.list[[i]]$chain_for_merge, "(TRB|IGH)"))
+      VDJ_airr <- subset(airr.list[[i]], stringr::str_detect(airr.list[[i]]$chain_for_merge, "(TRG|TRB|IGH)"))
       VDJ_airr <- VDJ_airr[,-c(which(names(VDJ_airr) %in% c("chain_for_merge")))]
       #rename to fit VGM
       names(VDJ_airr) <- paste0("VDJ_", names(VDJ_airr))
@@ -565,13 +587,28 @@ PlatypusDB_AIRR_to_VGM <- function(AIRR.input,
     VDJ.proc <- VDJ.proc.list
     VDJ.proc.list <- NULL
     if (VDJ.combine == T){ #processing all VDJ files together
-      VDJ.proc <- dplyr::bind_rows(VDJ.proc)
+      tryCatch({
+
+        for(i in 1:length(VDJ.proc)){ #setting all AIRR columns to character as this has caused issues in the past
+          for(j in 41:ncol(VDJ.proc[[i]])){
+            VDJ.proc[[i]][,j] <- as.character(VDJ.proc[[i]][,j])
+          }
+        }
+
+        VDJ.proc.comb <- dplyr::bind_rows(VDJ.proc)
+      }, error = function(e){
+        print(e)
+        cat("\n dplyr::bind_rows failed, returning list of VDJ dataframes")
+        VDJ.proc.comb <- VDJ.proc
+    })
+    } else {
+      VDJ.proc.comb <- VDJ.proc
     }
 
     if(verbose) cat("\n Done     ")
     if(verbose) print(Sys.time())
 
-    return(list("VDJ" = VDJ.proc,
+    return(list("VDJ" = VDJ.proc.comb,
                 "GEX" = "none",
                 "VDJ.GEX.stats" = out.stats,
                 "Running params" = params,
