@@ -1,54 +1,79 @@
-#'Circos plot for VDJ and VJ pairings
-#'
-#'@description Produces a Circos plot from the VDJ_GEX_matrix[[1]] output. Connects the V-alpha with the corresponding V-beta gene for each clonotype.
-#' @param VDJ For Platypus platypus.version v3, VDJ_GEX_matrix.output[[1]] has to be supplied. For Platypus V2 the output of the VDJ_GEX_integrate function is necessary
+#'Produces a Circos plot from the VDJ_analyze output. Connects the V-alpha with the corresponding V-beta gene for each clonotype.
+#' @param VGM The output of the VDJ_GEX_matrix function (VDJ_GEX_matrix.output[[1]]) has to be supplied. For Platypus v2: The output of the VDJ_GEX_integrate function (Platypus platypus.version v2). A list of data frames for each sample containing the clonotype information and cluster membership information.
 #' @param V.or.J Determines whether to plot the alpha beta gene pairing of the V or J genes. "V", "J" or "both" as possible inputs. Default: "both".
-#' @param label.threshold Minimal amount of clonotypes per gene neccessary to add a gene label to the sector. Default: 0.
-#' @param c.threshold Only clonotypes are considered with a frequency higher then c.threshold. Allows to filter for only highly expanded clonotypes.
-#' @param cell.level Logical, defines whether weight of connection should be based on number of clonotypes of number of cells. Default: number of clonotypes.
-#' @param clonotype.per.gene.threshold How many clonotypes are required to plot a sector for a gene. Filters the rows and colums of the final adjacency matrix.
 #' @param B.or.Tcells Specify whether B or T cells are being analyzed ("B" or "T"). If not specified, function attempts to decide based on gene names.
-#' @param c.count Show clonotype or cell count on Circos plot. Default = T.
-#' @param platypus.version Which platypus.version of platypus is being used. Default = "v3".
-#' @param filter1H1L Whether to filter the input VDJ.matrix in "v3" to only include cells with 1 VDJ and 1 VJ chain. Defaults to TRUE
-#' @return Returns list of plots. The first n elements contain the circos plot of the n datasets from the VDJ.analyze function. The n+1 element contains a list of the n adjancey matrices for each dataset.
+#' @param label.threshold Genes are only labeled if the count is larger then the label.threshold. By default all label.threshold = 0 (all genes are labeled).
+#' @param c.count.label Boolean, lets the user decide if the gene and count labels should be plotted or not. Default = T.
+#' @param c.count.label.size Determines the font size of the gene labels. By default the font size for count labels is 0.6.
+#' @param gene.label Boolean, lets the user decide if the gene labels should be plotted or not.
+#' @param gene.label.size Determines the font size of the gene labels. By default the labelsize is automatically adjusted to 0.7 for labels with two or less digits, 0.6 for labels between 2 and 6 digits, and 0.4 for all longer labels. A manually defined font size will be the same for all labels!
+#' @param platy.theme Allows plotting in the new "pretty" theme or the older "spiky" theme without group labels and radial arrangement of gene.labels. Default = "pretty".
+#' @param arr.col Data.frame with three columns where the first two indicate the names of genes, clonotypes or clusters to be connected, and the third corresponds to the color of the arrow. Default set to data.frame(c("dummy.clonotype"), c("dummy.cluster"), c("dummy.color")), so no arrow is drawn.
+#' @param arr.direction Either 1 or -1 and determines the direction of the arrow. Default=1.
+#' @param c.threshold Only clonotypes are considered with a frequency higher then c.threshold. Allows to filter for only highly expanded clonotypes.
+#' @param topX Filters for the top X clonotypes and only plots the respective gene combinations or cluster memberships.
+#' @param clonotype.per.gene.threshold How many clonotypes are required to plot a sector for a gene. Filters the rows and colums of the final adjacency matrix.
+#' @param filter1H1L Whether to filter the input VGM in "v3" to only include cells with 1 VDJ and 1 VJ chain. Defaults to TRUE
+#' @param clonotype.column Which column in VGM contains the clonotyping information? Default="clonotype_id_10X".
+#' @param cell.level Logical, defines whether weight of connection should be based on number of clonotypes of number of cells. Default: number of clonotypes.
+#' @param platypus.version Which platypus.version of platypus is being used. Default = v3. Set to v3 if VDJ_GEX_matrix.output[[1]] is used
+#' @return Returns a circos plot and a list object with the following elememts for N samples: [[1 to N]] The first N listelements corresponds to the recorded circos plots for N beeing the number or samples in the VGM. Since Circlize uses the R base plotting funciton, this is not a ggplot object but can still be replotted by calling the first list element. [[N+1]] Adjacency matrix forwarded to VDJ_circos(). This Matrix contains the counts and can be used for manual replotting using VDJ_circos directly. [[N+2]] Contains a named list with colors for each connection drawn and can be used for manual replotting using VDJ_circos directly. [[N+3]] Contains a named list with grouping information and can be used for manual replotting using VDJ_circos directly.
 #' @export
 #' @examples
 #' \dontrun{
-#'  plots <- VDJ_alpha_beta_Vgene_circos(Platypus::small_vgm[[1]]
-#'  , platypus.version="v3")
+#'  alpha_beta_VJgene <- VDJ_alpha_beta_Vgene_circos(vgm[[1]])
+#'  # print circos plot:
+#'  alpha_beta_VJgene[[1]]
 #'}
 
-VDJ_alpha_beta_Vgene_circos <- function(VDJ,
+VDJ_alpha_beta_Vgene_circos <- function(VGM,
                                         V.or.J,
                                         B.or.Tcells,
                                         label.threshold,
                                         c.threshold,
                                         cell.level,
                                         clonotype.per.gene.threshold,
-                                        c.count,
+                                        c.count.label,
+                                        c.count.label.size,
                                         platypus.version,
-                                        filter1H1L){
+                                        filter1H1L,
+                                        gene.label,
+                                        gene.label.size,
+                                        arr.col,
+                                        arr.direction,
+                                        topX,
+                                        platy.theme,
+                                        clonotype.column){
+
+  if(missing(V.or.J)){V.or.J <- "both"}
+  if(missing(label.threshold)){label.threshold <- 0}
+  if(missing(c.threshold)){c.threshold <- 0}
+  if(missing(cell.level)){cell.level <- F}
+  if(missing(clonotype.per.gene.threshold)){clonotype.per.gene.threshold <- 0}
+  if(missing(c.count.label)){c.count.label <- T}
+  if(missing(c.count.label.size)){c.count.label.size <- 0.6}
+  if(missing(platypus.version)){platypus.version ="v3"}
+  if(missing(filter1H1L)){filter1H1L <- T}
+  if(missing(gene.label)){gene.label <- T}
+  if(missing(gene.label.size)){gene.label.size <- "undef"}
+  if(missing(arr.col)){arr.col <- data.frame(c("dummy1"), c("dummy2"), c(""))}
+  if(missing(arr.direction)){arr.direction <- 1}
+  if(missing(topX)){topX <- "all"}
+  if(missing(platy.theme)){platy.theme <- "pretty"}
+  if(missing(clonotype.column)){clonotype.column <- "clonotype_id_10x"}
 
-if(missing(V.or.J)){V.or.J <- "both"}
-if(missing(label.threshold)){label.threshold <- 0}
-if(missing(c.threshold)){c.threshold <- 0}
-if(missing(cell.level)){cell.level <- F}
-if(missing(clonotype.per.gene.threshold)){clonotype.per.gene.threshold <- 0}
-if(missing(c.count)){c.count <- T}
-if(missing(platypus.version)){platypus.version ="v3"}
-if(missing(filter1H1L)){filter1H1L <- T}
 
 #define Variables
 
   clonotype <- NULL
   bk <- NULL
   VDJ.GEX.matrix <- NULL
+  VDJ.GEX_list <- NULL
   TRAV <- NULL
   TRBV <- NULL
   TRAJ <- NULL
   TRBJ <- NULL
-  plots <- NULL
+  plot <- NULL
   alpha_beta_Vgene <- NULL
   alpha_beta_Jgene <- NULL
   TRA <- NULL
@@ -62,19 +87,21 @@ if(missing(filter1H1L)){filter1H1L <- T}
   nm <- NULL
   group <- NULL
   level <- NULL
+  clonotypes_topX <- NULL
+  clonotype.frequency <- NULL
 
 
 if(platypus.version=="v3"){
 
   #########################################################################
-  clonotype <- "clonotype_id_10x"
+
 
   #swapping to a list to not change the whole function
-  bk <- VDJ
+  bk <- VGM
   VDJ.GEX.matrix <- list()
   VDJ.GEX.matrix[[1]] <- bk
   bk <- NULL
-  VDJ <- NULL
+  VGM <- NULL
 
   if(missing(B.or.Tcells)){
     for(i in 1:nrow(VDJ.GEX.matrix[[1]])){
@@ -93,17 +120,24 @@ if(platypus.version=="v3"){
     }
   }
 
-  plots <- list()
+  plot <- list()
 
-  #filter for 1H1L
+  # filter for 1H1L
 
   if(filter1H1L==T){
-    VDJ.GEX.matrix[[1]]<-VDJ.GEX.matrix[[1]][which((VDJ.GEX.matrix[[1]]$Nr_of_VDJ_chains==1)&(VDJ.GEX.matrix[[1]]$Nr_of_VJ_chains==1)),]
+    VDJ.GEX.matrix[[1]]<-VDJ.GEX.matrix[[1]][VDJ.GEX.matrix[[1]]$Nr_of_VDJ_chains==1 &VDJ.GEX.matrix[[1]]$Nr_of_VJ_chains==1,]
   }
 
-  #filter out clonotypes with less then c.threshold cells
-    VDJ.GEX.matrix[[1]] <- VDJ.GEX.matrix[[1]][which(VDJ.GEX.matrix[[1]]$clonotype_frequency >= c.threshold),]
+  clonotype.frequency <- paste0("clonotype_frequency_", stringr::str_split(clonotype.column, pattern="_")[[1]][3])
 
+  #filter out clonotypes with less then c.threshold cells
+  if(clonotype.frequency %in% colnames(VDJ.GEX.matrix[[1]])){ #check if 10x frequency already exists... will only be created after reclonotyping.
+    VDJ.GEX.matrix[[1]] <-VDJ.GEX.matrix[[1]][which(VDJ.GEX.matrix[[1]][[clonotype.frequency]] >= c.threshold),]
+    message(paste("Chosen clonotype.frequency column: ", clonotype.frequency))
+  }else{
+    VDJ.GEX.matrix[[1]] <-VDJ.GEX.matrix[[1]][which(VDJ.GEX.matrix[[1]][["clonotype_frequency"]] >= c.threshold),]
+    message(paste("Chosen clonotype.frequency column: ", "clonotype_frequency"))
+  }
 
   #replace empty fields "" with "None"
   VDJ.GEX.matrix[[1]]$VJ_vgene[which(VDJ.GEX.matrix[[1]]$VJ_vgene == "")] <- "None"
@@ -120,9 +154,19 @@ if(platypus.version=="v3"){
   #split up into samples to plot individually
   VDJ.GEX_list <- list()
   for (i in 1:length(unique(VDJ.GEX.matrix[[1]]$sample_id))){
+    print(unique(VDJ.GEX.matrix[[1]]$sample_id)[i])
     VDJ.GEX_list[[i]] <- VDJ.GEX.matrix[[1]][which(VDJ.GEX.matrix[[1]]$sample_id== unique(VDJ.GEX.matrix[[1]]$sample_id)[i]),]
   }
 
+  # filter topX clonotypes
+
+  if(topX != "all"){
+    for(k in 1:length(VDJ.GEX_list)){
+      clonotypes_topX <- names(utils::head(sort(table(VDJ.GEX_list[[k]][[clonotype.column]]),decreasing = T),topX))
+      #filter and keep only cells of topX clonotypes
+      VDJ.GEX_list[[k]] <- VDJ.GEX_list[[k]][which(VDJ.GEX_list[[k]][[clonotype.column]] %in% clonotypes_topX),]
+    }
+  }
 
   for (i in 1:length(VDJ.GEX_list)){
 
@@ -171,6 +215,13 @@ if(platypus.version=="v3"){
     colnames(Vgene_usage_matrix[[k]]) <- unique(TRB)
   }
 
+
+
+  if(cell.level == F){
+    message("WARNING: If clonotype strategy is not based on unique V or J genes per clonotype, this setting [cell.level=F] might be questionable. One clonotype might then be represented in several Circos connections between V or J genes. The names of genes of simulatneously used chains will be pasted together.")
+    message(paste("Chosen clonotype column: ", clonotype.column))
+  }
+
   for (k in 1:length(VDJ.GEX_list)){
 
     #create dummy df which will contain the counts for each combination
@@ -183,11 +234,7 @@ if(platypus.version=="v3"){
     }else{
       #print("---")
       message(paste0("Processing sample ", k))
-      #print("WARNING: If clonotype strategy is not based on unique V or J genes per clonotype, this setting [cell.level=F] might be questionable. One clonotype might then be represented in several Circos connections between V or J genes. The names of genes of simulatneously used chains will be pasted together.")
-      #print(paste("Chosen clonotype column: ", clonotype))
-      #print("WARNING: If Circos plotting error occurs: Maybe your `gap.degree` is too large so that there is no space to allocate sectors -> You might want to increase clonotype.per.gene.threshold to reduce number of sectors in your Circos plots")
-
-      dummy <- as.data.frame(unique(paste(VDJ.GEX_list[[k]][[clonotype]],VDJ.GEX_list[[k]]$alpha_beta_Vgene, sep="/and/")))
+      dummy <- as.data.frame(unique(paste(VDJ.GEX_list[[k]][[clonotype.column]],VDJ.GEX_list[[k]]$alpha_beta_Vgene, sep="/and/")))
       colnames(dummy) <- c("pasted")
       dummy$clonotype <- stringr::str_split_fixed(dummy$pasted, "/and/", 2)
       dummy$gene <- stringr::str_split_fixed(dummy$pasted, "/and/", 2)[,2]
@@ -196,7 +243,7 @@ if(platypus.version=="v3"){
       colnames(dummy_Vgene_df[[k]]) <- c("gene", "count")
 
 
-      dummy <- as.data.frame(unique(paste(VDJ.GEX_list[[k]][[clonotype]],VDJ.GEX_list[[k]]$alpha_beta_Jgene, sep="/and/")))
+      dummy <- as.data.frame(unique(paste(VDJ.GEX_list[[k]][[clonotype.column]],VDJ.GEX_list[[k]]$alpha_beta_Jgene, sep="/and/")))
       colnames(dummy) <- c("pasted")
       dummy$clonotype <- stringr::str_split_fixed(dummy$pasted, "/and/", 2)[,1]
       dummy$gene <- stringr::str_split_fixed(dummy$pasted, "/and/", 2)[,2]
@@ -307,14 +354,20 @@ if(platypus.version=="v3"){
     # Set grouping factors based on previously defined levels. Order of levels defines order of groups in Circos plot.
     group = factor(group[sample(length(group), length(group))], levels = levels)
 
-    plots[[i]] <- VDJ_circos(Vgene_usage_matrix[[i]], group = group, grid.col=grid.col, label.threshold = label.threshold, c.count = c.count)
+    VDJ_circos(Vgene_usage_matrix[[i]], group = group, grid.col=grid.col, label.threshold = label.threshold, c.count.label = c.count.label, gene.label = gene.label, gene.label.size = gene.label.size, c.count.label.size = c.count.label.size, arr.col = arr.col, arr.direction = arr.direction, platy.theme = platy.theme)
+    circos.recorded <- grDevices::recordPlot()
+    plot[[i]] <- circos.recorded
   }
-  plots[[i+1]] <- Vgene_usage_matrix
+
+  plot[[i+1]] <- Vgene_usage_matrix
+  plot[[i+2]] <- grid.col
+  plot[[i+3]] <- group
+
 
 }else if(platypus.version=="v2"){
+#### Platypus Version 2 ####
 
 
-  ##########################################################################
   if(missing(B.or.Tcells)){
     for(i in 1:nrow(VDJ[[1]])){
       if(substr(VDJ[[1]]$HC_vgene[[i]],start=1, stop = 2)=="IG"){
@@ -332,7 +385,7 @@ if(platypus.version=="v3"){
     }
   }
 
-      plots <- list()
+      plot <- list()
 
       #filter out clonotypes with less then c.threshold cells
       for(i in 1:length(VDJ)){
@@ -500,11 +553,11 @@ if(platypus.version=="v3"){
         # Set grouping factors based on previously defined levels. Order of levels defines order of groups in Circos plot.
         group = factor(group[sample(length(group), length(group))], levels = levels)
 
-        plots[[i]] <- VDJ_circos(Vgene_usage_matrix[[i]], group = group, grid.col=grid.col, label.threshold = label.threshold, c.count = c.count)
+        plot[[i]] <- VDJ_circos(Vgene_usage_matrix[[i]], group = group, grid.col=grid.col, label.threshold = label.threshold, c.count.label = c.count.label)
       }
-      plots[[i+1]] <- Vgene_usage_matrix
+      plot[[i+1]] <- Vgene_usage_matrix
 }else{
   stop("Please specify platypus platypus.version as either v2 or v3. v3 in case of VDJ_GEX_matrix input")
   }
-  return(plots)
+  return(plot)
 }
