@@ -10,6 +10,7 @@
 #' @param match.by string, represents the method by which to match the antigen data and integrate it into the VDJ/VDJ.GEX.matrix[[1]] object. 'clonotype' will match by 'clonotype_id' (needs to be present in the antigen data), 'clonotype.v3' will match by v3 cellranger clonotypes (you need a v3_clonotypes column in the VDJ/VDJ.GEX.matrix[[1]], 'cdr3.aa' by VDJ and VJ cdr3s amino acid sequences, 'cdrh3.aa' by VDJ cdr3s amino acid sequences, 'VDJ.VJ.aa' by full VDJ and VJ aa sequences, 'VDJ.VJ.nt' by trimmed nt VDJ and VJ sequences (must run VDJ_call_MIXCR first on the VDJ),'cdr3.nt' by VDJ and VJ cdr3s as nucleotides, 'cdrh3.nt.' by VDJ cdr3s as nucleotides, 'absolut' will match the VDJ_cdr3s_aa with the CDR3 column in Absolut! datasets.
 #' @param matching.type string, either 'exact' for exact sequence matching if the match.by parameter is a sequence type, or 'homology' for homology matching (matches if the Levehnstein distance is less than the distance.threshold parameter).
 #' @param distance.threshold integer, maximum string distance value by which to match sequences in the antigen data and sequences in the VDJ object (to further integrate the antigen data).
+#' @param cores Number of cores to use for parallel computations. Defaults to number of available cores. Setting this parameter is good practice on clusters.
 #' @param sample.id boolean, if T then will also match by the 'sample_id' column in the antigen dataframes.
 #' @param aberrant.chosen.sequences boolean, if T will add a column of the chosen aberrant sequences (which matched a sequence in the antigen data) if matching by sequence (and VDJ.VJ.1chain=F).
 #' @param output.format string, 'vgm' - returns the full VDJ object, 'dataframe.per.sample' - list of VDJ dataframes for each sample.
@@ -18,7 +19,7 @@
 
 #' @examples
 #' \dontrun{
-#' VDJ_antigen_integrate_v2(VDJ,antigen.directory.list=antigen.directory.list,
+#' VDJ_antigen_integrate(VDJ,antigen.directory.list=antigen.directory.list,
 #' antigen.feature=c('elisa', 'affinity'),VDJ.VJ.1chain=T,
 #' match.by='clonotype',sample.id=T, output.format='vgm')
 #'}
@@ -32,6 +33,7 @@ VDJ_antigen_integrate <- function(VDJ,
                                  match.by,
                                  matching.type,
                                  distance.threshold,
+                                 cores,
                                  sample.id,
                                  aberrant.chosen.sequences,
                                  output.format){
@@ -48,6 +50,21 @@ VDJ_antigen_integrate <- function(VDJ,
   if(missing(sample.id)) sample.id <- T
   if(missing(aberrant.chosen.sequences)) aberrant.chosen.sequences <- T
   if(missing(output.format)) output.format <- 'vgm'
+  if(missing(cores))  cores <- parallel::detectCores()
+
+
+
+  #Detecting operating system for correct parallel apply function
+  operating.system <- "Windows" #setting default to Windows as parlapply will work on all systems, but will only be time efficient on windows.
+    switch(Sys.info()[['sysname']],
+           Windows= {message("Windows system detected")
+             operating.system <- "Windows"},
+           Linux  = {message("Linux system detected")
+             operating.system <- "Linux"},
+           Darwin = {message("MAC system detected")
+             operating.system <- "Darwin"})
+
+
 
   get_sequence_combinations <- function(x, y, split.x, split.y, split.by=';', collapse.by=';'){
    if(split.x==T) x <- stringr::str_split(x, split.by ,simplify=T)[1,]
@@ -148,12 +165,21 @@ VDJ_antigen_integrate <- function(VDJ,
     sample_sequences <- unique(sample_sequences)
     antigen_sequences <- antigen_df[,sequence_column]
 
-    distance_matrix <- stringdist::stringdistmatrix(sample_sequences, antigen_sequences, method='lv')
 
     if(matching_type=='exact'){
-      matched_indices_from_distance_matrix <- lapply(1:nrow(distance_matrix), function(x) if(min(distance_matrix[x, ]) == 0) which(distance_matrix[x, ]==min(distance_matrix[x, ])) else NA)
-      }else if(matching_type=='homology'){
-      matched_indices_from_distance_matrix <- lapply(1:nrow(distance_matrix), function(x) if(min(distance_matrix[x, ]) <= distance_threshold) which(distance_matrix[x, ]==min(distance_matrix[x, ])) else NA)
+      #requireNamespace('parallel')
+      if(operating.system %in% c("Darwin", "Linux")){
+      matched_indices <- parallel::mclapply(sample_sequences, mc.cores = cores, FUN = function(x) {if(any(antigen_sequences==x)) which(antigen_sequences==x) else NA})
+      } else if(operating.system == "Windows"){
+        doParallel::stopImplicitCluster() #close any open clusters
+        cl <- parallel::makeCluster(cores)  #open cluster for parallel computing
+        matched_indices <- parallel::parLapply(cl = cl, sample_sequences, fun = function(x) {if(any(antigen_sequences==x)) which(antigen_sequences==x) else NA})
+      }
+      distance_matrix <- stringdist::stringdistmatrix(sample_sequences, antigen_sequences, method='lv')
+
+    }else if(matching_type=='homology'){
+      distance_matrix <- stringdist::stringdistmatrix(sample_sequences, antigen_sequences, method='lv')
+      matched_indices <- lapply(1:nrow(distance_matrix), function(x) if(min(distance_matrix[x, ]) <= distance_threshold) which(distance_matrix[x, ]==min(distance_matrix[x, ])) else NA)
     }
 
 
@@ -162,7 +188,7 @@ VDJ_antigen_integrate <- function(VDJ,
         sample_df$aberrant_chosen_sequences <- rep(NA, nrow(sample_df))
       }
       aberrant_indices <- which(sample_df$Nr_of_VDJ_chains==2 | sample_df$Nr_of_VJ_chains==2)
-      matched_feature_values <- lapply(matched_indices_from_distance_matrix, function(x) if(!is.na(x[1])) unlist(antigen_df[,features[[i]]][x]) else NA)
+      matched_feature_values <- lapply(matched_indices, function(x) if(!is.na(x[1])) unlist(antigen_df[,features[[i]]][x]) else NA)
 
       #matched_max_indices <- mapply(function(x,y) if(inherits(y[1])=='numeric') {if(!is.na(x[1])) x[which.max(y)] else NA} else if(inherits(y)!='numeric{'){ x[1]}, matched_indices_from_distance_matrix, matched_feature_values)
       matched_values <- sapply(matched_feature_values, function(x) if(inherits(x[1],'numeric')) {if(!is.na(x[1])) x[which.max(x)] else NA} else if(!inherits(x,'numeric')){if(!is.na(x[1])) paste0(x, collapse=';') else NA})
@@ -196,7 +222,7 @@ VDJ_antigen_integrate <- function(VDJ,
           if(aberrant_chosen_sequences){
             sample_df$aberrant_chosen_sequences[intersect(pre_matched_indices, aberrant_indices)] <- paste(sample_df$aberrant_chosen_sequences[intersect(pre_matched_indices, aberrant_indices)],'/', sequence)
           }
-          sample_df$new_feature[intersect(pre_matched_indices, aberrant_indices)] <- paste(sample_df$new_feature[intersect(pre_matched_indices, aberrant_indices)], '/', matched_values[j])
+          sample_df$new_feature[intersect(pre_matched_indices, aberrant_indices)] <- unique(c(sample_df$new_feature[intersect(pre_matched_indices, aberrant_indices)], matched_values[j]))
 
         }
       }
@@ -222,9 +248,6 @@ VDJ_antigen_integrate <- function(VDJ,
   }
 
 
-  VDJ.matrix <- VDJ
-  VDJ <- NULL
-
   antigen_dfs <- list()
   if(inherits(antigen.data.list[[1]],'data.frame')){
     antigen_dfs <- antigen.data.list
@@ -236,20 +259,20 @@ VDJ_antigen_integrate <- function(VDJ,
   }
 
   if(VDJ.VJ.1chain==T){
-    VDJ.matrix <- VDJ.matrix[which(VDJ.matrix$Nr_of_VDJ_chains==1 & VDJ.matrix$Nr_of_VJ_chains==1),]
+    VDJ <- VDJ[which(VDJ$Nr_of_VDJ_chains==1 & VDJ$Nr_of_VJ_chains==1),]
   }
 
   sample_dfs <- list()
 
   if(sample.id==T){
-    repertoire.number <- unique(VDJ.matrix$sample_id)
+    repertoire.number <- unique(VDJ$sample_id)
 
     for(i in 1:length(repertoire.number)){
-      sample_dfs[[i]] <- VDJ.matrix[which(VDJ.matrix$sample_id==repertoire.number[i]),]
+      sample_dfs[[i]] <- VDJ[which(VDJ$sample_id==repertoire.number[i]),]
     }
 
   }else{
-    sample_dfs[[1]] <- VDJ.matrix
+    sample_dfs[[1]] <- VDJ
   }
 
 
@@ -340,9 +363,9 @@ VDJ_antigen_integrate <- function(VDJ,
     return(sample_dfs)
 
   }else if(output.format=='vgm'){
-    if(sample.id) VDJ.matrix <- do.call('rbind',sample_dfs)
-    if(!sample.id) VDJ.matrix <- sample_dfs[[1]]
+    if(sample.id) VDJ <- do.call('rbind',sample_dfs)
+    if(!sample.id) VDJ <- sample_dfs[[1]]
 
-    return(VDJ.matrix)
+    return(VDJ)
   }
 }
